package com.example.service;

import com.example.entity.User;
import com.example.repository.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;


@Service
public class UserService {

    @Autowired
    private UserRepository repository;

    Object target;
    Logger logger = LoggerFactory.getLogger(UserService.class);

    @Async
    public CompletableFuture<List<User>> saveUsers(MultipartFile file) throws Exception {
        long start = System.currentTimeMillis();
        List<User> users = parseCSVFile(file);
        logger.info("saving list of users of size {}", users.size(), "" + Thread.currentThread().getName());
        users = repository.saveAll(users);
        long end = System.currentTimeMillis();
        logger.info("Total time {}", (end - start));
        return CompletableFuture.completedFuture(users);
    }

    // method that gets the user by id @Async annotation lets us run the method in a separate thread every time it is called.
    @Async
    public CompletableFuture<User> findUserById(int id) {
        logger.info("get user with id " + id + " by " + Thread.currentThread().getName());
        User user = repository.findById(id).get();
        return CompletableFuture.completedFuture(user);
    }

    // save and get the data through a single controller asynchronously.
    @Async
    public void saveUserById(int id) {
        User user = new User(id, "abc", "xyz@mail.com", Integer.toString(id));
        repository.save(user);
        logger.info("save user with id " + id + " by " + Thread.currentThread().getName());
    }

    @Async
    public CompletableFuture<User> getUserById(int id) {
        User user = null;
        try{
            user = repository.findById(id).get();
            return CompletableFuture.completedFuture(user);
        }catch (Exception ex){
            logger.error(ex.getMessage());
        }finally {
            logger.info("get user with id " + id + " by " + Thread.currentThread().getName());
        }
        return null;
    }

    private List<User> parseCSVFile(final MultipartFile file) throws Exception {
        final List<User> users = new ArrayList<>();
        try {
            try (final BufferedReader br = new BufferedReader(new InputStreamReader(file.getInputStream()))) {
                String line;
                br.readLine();
                while ((line = br.readLine()) != null) {
                    final String[] data = line.split(",");
                    final User user = new User();
                    user.setName(data[0]);
                    user.setEmail(data[1]);
                    user.setGender(data[2]);
                    users.add(user);
                }
                return users;
            }
        } catch (final IOException e) {
            logger.error("Failed to parse CSV file {}", e);
            throw new Exception("Failed to parse CSV file {}", e);
        }
    }


}
